-- TankPoints Mod
TankPoints_Version = "0.2";
TankPoints_Author = "Whitetooth";

-- Default Values

-- Function Hooks
local lOriginal_PaperDollFrame_SetResistances;

-- OnLoad
function TankPoints_OnLoad()
	-- Slash commands
	SlashCmdList["TANKPOINTSCOMMAND"] = TankPoints_SlashHandler;
	SLASH_TANKPOINTSCOMMAND1 = "/tankpoints";
	
	-- Register events
	this:RegisterEvent("UNIT_LEVEL");
	this:RegisterEvent("UNIT_RESISTANCES");
	this:RegisterEvent("UNIT_STATS");
	this:RegisterEvent("UNIT_DEFENSE");
	this:RegisterEvent("UNIT_MAXHEALTH");
	this:RegisterEvent("UNIT_AURA");
	
	--Hook PaperDollFrame_SetResistances
	if (not lOriginal_PaperDollFrame_SetResistances) then		
		lOriginal_PaperDollFrame_SetResistances = PaperDollFrame_SetResistances;
		PaperDollFrame_SetResistances = PaperDollFrame_SetResistances_New;
	end
end

function TankPoints_SlashHandler(msg)
	local index, value;
	if (not msg or msg == "") then --Show Help
	else
		local command=strlower(msg);
		if (command == "test") then
		end
	end
end

function TankPointsFrame_OnShow()
	TankPointsFrame_SetPoints()
end

function TankPointsFrame_OnEvent()
	TankPointsFrame_SetPoints()
end

function TankPointsFrame_SetPoints()
	local unit = "player";
	local label = getglobal("TankPointsFrameLabel");
	local text = getglobal("TankPointsFrameStatText");
	local frame = getglobal("TankPointsFrame");
	label:SetText("TankPoints:");
	
	local maxHealth = UnitHealthMax(unit);
	local playerLevel = UnitLevel(unit);
	local base, effectiveArmor, armor, posBuff, negBuff = UnitArmor(unit);
	local armorReduction = effectiveArmor/((85 * playerLevel) + 400);
	armorReduction = (armorReduction/(armorReduction + 1));
	local dodge = GetDodgeChance();
	local parry = GetParryChance();
	local block = GetBlockChance();
	local base, modifier = UnitDefense(unit);
	local defense = base + modifier;
	local critReduction = (defense - playerLevel * 5) * 0.04 * 0.01;
	local mobCrit = max(0, 0.05 - critReduction);
	local mobDps = 1;
	
	local totalReduction = 1 - (mobCrit * 2 + (1 - mobCrit) * (1 - dodge * 0.01) * (1 - parry * 0.01)) * (1 - armorReduction);
	local tankPoints = maxHealth / (mobDps * (mobCrit * 2 + (1 - mobCrit) * (1 - dodge * 0.01) * (1 - parry * 0.01)) * (1 - armorReduction));
	
	text:SetText(format("%5.0f", tankPoints));
	
	-- Set the tooltip text
	frame.tooltip = HIGHLIGHT_FONT_COLOR_CODE.."TankPoints "..format("%5.3f", tankPoints)..FONT_COLOR_CODE_CLOSE;
	
	-- Set the subtooltip text
	TANKPOINTS_TOOLTIP_SUBTEXT = "Armor: %d (%2.3f%%)\nDodge: %2.3f%%\nParry: %2.3f%%\nCrit Reduction: %1.2f%%\nTotal Reduction: %2.3f%%";
	frame.tooltipSubtext = format(TANKPOINTS_TOOLTIP_SUBTEXT, effectiveArmor, armorReduction * 100, dodge, parry, critReduction * 100, totalReduction * 100);
end

function PaperDollFrame_SetResistances_New()
	for i=1, NUM_RESISTANCE_TYPES, 1 do
		local resistance;
		local positive;
		local negative;
		local base;
		local text = getglobal("MagicResText"..i);
		local frame = getglobal("MagicResFrame"..i);
		
		base, resistance, positive, negative = UnitResistance("player", frame:GetID());

		local resistanceName = getglobal("RESISTANCE"..(frame:GetID()).."_NAME");
		frame.tooltip = resistanceName.." "..RESISTANCE_LABEL.." "..resistance;

		-- resistances can now be negative. Show Red if negative, Green if positive, white otherwise
		if( abs(negative) > positive ) then
			text:SetText(RED_FONT_COLOR_CODE..resistance..FONT_COLOR_CODE_CLOSE);
		elseif( abs(negative) == positive ) then
			text:SetText(resistance);
		else
			text:SetText(GREEN_FONT_COLOR_CODE..resistance..FONT_COLOR_CODE_CLOSE);
		end

		if ( positive ~= 0 or negative ~= 0 ) then
			-- Otherwise build up the formula
			frame.tooltip = frame.tooltip.. " ( "..HIGHLIGHT_FONT_COLOR_CODE..base;
			if( positive > 0 ) then
				frame.tooltip = frame.tooltip..GREEN_FONT_COLOR_CODE.." +"..positive;
			end
			if( negative < 0 ) then
				frame.tooltip = frame.tooltip.." "..RED_FONT_COLOR_CODE..negative;
			end
			frame.tooltip = frame.tooltip..FONT_COLOR_CODE_CLOSE.." )";
		end

		local unitLevel = UnitLevel("player");
		unitLevel = max(unitLevel, 20);
		-- Added by Whitetooth
		local spellReduction = 75 * min(resistance, unitLevel * 5) / (unitLevel * 5);
		frame.tooltip = frame.tooltip.." ["..format("%2.2f", spellReduction).."%]";
		
		local magicResistanceNumber = resistance/unitLevel;
		if ( magicResistanceNumber > 5 ) then
			resistanceLevel = RESISTANCE_EXCELLENT;
		elseif ( magicResistanceNumber > 3.75 ) then
			resistanceLevel = RESISTANCE_VERYGOOD;
		elseif ( magicResistanceNumber > 2.5 ) then
			resistanceLevel = RESISTANCE_GOOD;
		elseif ( magicResistanceNumber > 1.25 ) then
			resistanceLevel = RESISTANCE_FAIR;
		elseif ( magicResistanceNumber > 0 ) then
			resistanceLevel = RESISTANCE_POOR;
		else
			resistanceLevel = RESISTANCE_NONE;
		end
		
		frame.tooltipSubtext = format(RESISTANCE_TOOLTIP_SUBTEXT, resistanceName, unitLevel, resistanceLevel);
		
		-- Added by Whitetooth
		local maxHealth = UnitHealthMax("player");
		local resistTP = maxHealth / (1 - spellReduction / 100);
		frame.tooltipSubtext = frame.tooltipSubtext.."\n"..LIGHTYELLOW_FONT_COLOR_CODE..resistanceName.." Tank Points: "..format("%5.0f", resistTP)..FONT_COLOR_CODE_CLOSE;
	end
end